function [curr_voltage,AP]=LIF_one_step_multiple_neurons(curr_voltage,Input,params)
%LIF_ONE_STEP_MULTIPLE_NEURONS Runs one step of simulation - LIF neurons
%
% Performs one step of LIF simulation. Uses curr_voltage as the current
% voltage of the model neurns. Updates curr_voltage and records which
% neurons (if any) fired in AP.
%
% The simulation steps are scaled by params.num_steps_per_ms. If each steps
% "takes" 1 ms then the value should be 1.
%
% Please note that even though the neurons are represented in a single
% vector they can still represent a 2D network of neurons. E.g. assume we
% have a network of 4 rows by 3 columns of neurons - we can represent them
% by a single vector of 12 neurons. if this matrix is named D then we can
% transform it to a vector by D(:) and retrieve it by reshape(D(:),4,3)
%
% Written by Omri Perez, 2012, omriperez(at)gmail(dot)com

%% Model Parameters
if nargin<3
    params.tau=15;                 % time constant (ms)
    params.input_gain=100;         % gain to scale input (normalized xcorr is [-1,1])
    params.noise_sd=0;             % number of SDs for gaussian input noise
    params.AP_threshold=-30;       % AP threshold (mV)
    params.num_steps_per_ms=1; % num of steps per ms
    params.lateral_inhibition=200;   % gain to scale lateral inhibition
    params.rest_potential=-70;     % resting state potential (mV)
end

%% Set important variables
I_ext=params.input_gain .* ... scale ->
    (Input+randn(size(Input))*params.noise_sd); %input + noise
tau=params.tau*params.num_steps_per_ms; %update tau constant for step size

AP=zeros(size(Input)); %action potential (AP) vector
%% One Step Integration with Euler method

curr_firing=curr_voltage>params.AP_threshold; %which neurons currently fire?
curr_voltage(curr_firing)=params.rest_potential; %reset their voltage
AP(curr_firing)=1; %mark AP for them

%not firing - update voltage:
curr_voltage(~curr_firing)=curr_voltage(~curr_firing)+ ... %the current voltage
    (params.rest_potential-curr_voltage(~curr_firing))./tau+... %minus leak
    I_ext(~curr_firing)./tau -... %plus (noisy?) input
    sum(AP)*params.lateral_inhibition/tau; %minus lateral inhibition

end

